"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureWorkspaceGenerator = void 0;
const tslib_1 = require("tslib");
const seeds_1 = require("../../../seeds");
const utils_1 = require("../../../utils");
const seed_1 = require("../../actions/seed");
const messages_1 = require("../../messages");
const tasks_1 = require("../../tasks");
const applyRuntimeArgs = (prevAns = {}) => {
    if (!prevAns.targetDirectory) {
        prevAns.targetDirectory = utils_1.argv.targetDirectory || utils_1.env.GENESISLCAP_TARGET_DIRECTORY;
    }
    return prevAns;
};
const validatePrevAns = (prevAns = {}) => {
    const { targetDirectory, workspaceSeed } = prevAns;
    if (!(0, utils_1.directoryExists)(targetDirectory)) {
        throw new Error(`Target directory does not exist: ${targetDirectory}`);
    }
    if (!workspaceSeed) {
        throw new Error(`Invalid seed information, unable to configure. Seed: ${workspaceSeed}`);
    }
};
exports.configureWorkspaceGenerator = {
    description: (0, messages_1.generatorDescription)('Configure a local workspace.'),
    prompts(inquirer, prevAns = {}) {
        var _a, _b, _c;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            prevAns = applyRuntimeArgs(prevAns);
            const { targetDirectory = prevAns.targetDirectory } = yield inquirer.prompt([
                {
                    name: 'targetDirectory',
                    type: 'select-directory',
                    message: 'Workspace directory',
                    when: !prevAns.targetDirectory,
                    basePath: utils_1.basePath,
                },
            ]);
            /**
             * Ensure targetDirectory is present in prevAns
             */
            // eslint-disable-next-line require-atomic-updates
            prevAns.targetDirectory = targetDirectory;
            /**
             * Get the seed's genx api / config file
             */
            const configFile = (0, seeds_1.getSeedConfigFile)(targetDirectory);
            /**
             * Load prevAns from the seed via the configFile api and merge. These will be empty the first time, ie. creating a
             * new workspace chained to this configure generator. Devs may also run configure workspace in isolation, and will
             * have to chose a existing workspace from their filesystem, which should contain the prevAns from the last CLI
             * interaction with the seed itself. Answers are committed to source control so other devs can run future automated
             * CLI tasks.
             */
            const seedAns = (_b = (_a = configFile.answers()) === null || _a === void 0 ? void 0 : _a.read()) !== null && _b !== void 0 ? _b : {};
            const hasSeedAns = Object.keys(seedAns).length > 0;
            if (hasSeedAns) {
                /**
                 * Remap the original seed option to gain access to the configure method linked to it
                 */
                const seedName = (_c = seedAns.workspaceSeed) === null || _c === void 0 ? void 0 : _c.name;
                let sourceSeedOption = null;
                if (seedName) {
                    const seedList = yield (0, tasks_1.downloadSeedList)('workspaces');
                    sourceSeedOption = seedList.find((w) => w.name === seedName);
                }
                if (sourceSeedOption) {
                    seedAns.workspaceSeed = sourceSeedOption;
                }
                Object.assign(prevAns, seedAns);
            }
            validatePrevAns(prevAns);
            const seedPrompts = configFile.prompts();
            const seedPromptAns = yield seedPrompts(inquirer, prevAns);
            return seedPromptAns;
        });
    },
    actions: (answers) => [
        {
            type: seed_1.SeedActionType.configure,
            get directory() {
                return answers.targetDirectory;
            },
            get seed() {
                return answers.workspaceSeed;
            },
            abortOnFail: true,
        },
        {
            type: seed_1.SeedActionType.setEnvVars,
            get directory() {
                return answers.targetDirectory;
            },
        },
    ],
};
